/**
 * @file
 * Decrypts cipher texts in a list.
 */
(function($, Drupal, once) {
  const ProcBehavior = {
    async attach(context, settings) {
      if (context.id !== '') {
        const procItems = document.querySelectorAll('[id^="proc-list-item"]');
        const cipherTexts = [];
        const procFormat = 'binary';
        procItems.forEach((procItem, procItemIndex) => {
          const cipherText = procItem.getAttribute(
            'data-proc-list-item-cipher-text',
          );
          cipherTexts.push(cipherText);
        });
        const _navigator = {};
        for (const i in navigator) _navigator[i] = navigator[i];
        const passowrdSeesionCachingSalt = JSON.stringify(_navigator);
        const cachedPassword = sessionStorage.getItem(
          `proc.password_cache.key_user_id.${drupalSettings.user.uid}`,
        );
        if (!cachedPassword) {
          // There is no cached password.
          const dialogDiv = $(document.createElement('div'));
          dialogDiv.attr('id', 'proc-decrypt-dialog');
          const passwordField = $(document.createElement('input'));
          passwordField.attr('type', 'password');
          passwordField.attr('id', 'password');
          const passwordLabel = $(document.createElement('label'));
          passwordLabel.attr('for', 'password');
          passwordLabel.html('Password');
          passwordField[0].addEventListener('input', function(evt) {
            decryptListItems(this.value);
          });
          dialogDiv.append(passwordLabel);
          dialogDiv.append(passwordField);
          dialogDiv.dialog();
        }

        /**
         * Decrypts the private key.
         *
         * Decrypts the private key given a correct passphrase of symmetric encryption.
         *
         * @param password
         *   Password that was typed in or taken from cached.
         */
        async function decryptPrivateKey(password) {
          const passphrase = drupalSettings.proc.proc_pass.concat(password);
          const privateKey = await openpgp.readPrivateKey({
            armoredKey: drupalSettings.proc.proc_privkey,
          });
          const decryptedPrivateKey = await openpgp
            .decryptKey({
              privateKey,
              passphrase,
            })
            .catch(function(err) {
              const decryptionLinkElement = $('a#decryption-link');
              $('form#-proc-decrypt-to-file').prepend(
                `<div class="messages error">${Drupal.t(err)}</div>`,
              );
            });
          return decryptedPrivateKey;
        }

        /**
         * Decrypts all listed items.
         *
         * Decrypts all listed items with the decrypted private key.
         *
         * @param decryptedPrivateKey
         *   Symmetrically decrypted private key.
         */
        async function decryptAllItems(decryptedPrivateKey) {
          for (const cipherText of cipherTexts) {
            const cipherTextIndex = cipherTexts.indexOf(cipherText);

            const message = await openpgp.readMessage({
              armoredMessage: cipherText,
            });
            const decrypted = await openpgp
              .decrypt({
                decryptionKeys: decryptedPrivateKey,
                message,
                format: procFormat,
              })
              .catch(function(err) {
                $('.messages').after(
                  `<div class="messages messages--error proc--error" id="proc-decrypting-status">${Drupal.t(
                    err,
                  )}</div>`,
                );
              });

            if (decrypted) {
              const string = new TextDecoder().decode(decrypted.data);
              document.querySelectorAll('[id^="proc-list-item"]')[
                cipherTextIndex
              ].innerText = string;
            }
          }
        }

        /**
         * Decrypts all listed items from uncached password dialog.
         *
         * Decrypts all listed items with a correct passphrase of symmetric encryption.
         *
         * @param password
         *   Passphrase for symmetrical decryption of private key.
         */
        async function decryptListItems(password) {
          const decryptedPrivateKey = await decryptPrivateKey(password);
          if (decryptedPrivateKey) {
            decryptAllItems(decryptedPrivateKey);
            // Close dialog.
            $('#proc-decrypt-dialog').dialog('close');
          }
        }

        if (cachedPassword) {
          try {
            await (async () => {
              const messageSource = await openpgp.readMessage({
                armoredMessage: sessionStorage.getItem(
                  `proc.password_cache.key_user_id.${drupalSettings.user.uid}`,
                ),
              });
              const { data: secretPassString } = await openpgp.decrypt({
                message: messageSource,
                passwords: [passowrdSeesionCachingSalt],
              });

              const decryptedPrivateKey = await decryptPrivateKey(
                secretPassString,
              );

              decryptAllItems(decryptedPrivateKey);
            })();
          } catch (err) {
            // This error may happen because of a change in the browser fingerprint.
            console.info(err);
          }
        }
      }
    },
  };
  Drupal.behaviors.ProcBehavior = ProcBehavior;
})(jQuery, Drupal, once);
